/* Tabela INSCRMUNICIP: Item do Lanamento de Estoque */
/* Descrio: A trigger insere registros na tabela LOGOPERA a partir de alteraes na tabela INSCRMUNICIP */
/* Objetivo: Gerar um log que vai registrar as alteraes na tabela INSCRMUNICIP utilizando a tabela Geral de Logs de Operaes */


IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'LOG_GER_INSCRMUNICIP'  AND
                 TYPE   = 'TR'                    AND
                 U.UID  = O.UID                   AND
                 U.NAME = USER)
DROP TRIGGER LOG_GER_INSCRMUNICIP
GO

CREATE TRIGGER LOG_GER_INSCRMUNICIP
ON INSCRMUNICIP
FOR DELETE, INSERT, UPDATE
AS
   DECLARE @V_USUARIO      VARCHAR(40);
   DECLARE @V_TIPO         VARCHAR(1);
   DECLARE @V_ITPESQ       VARCHAR(40);
   DECLARE @V_DESC         VARCHAR(300);
   DECLARE @V_CDFILLOGOPER VARCHAR(4);
   DECLARE @V_DTOPER       DATETIME;

/*VARIVEIS DO CURSOR C_CURSOR*/

   DECLARE @V_ID_COMANDO       VARCHAR(1);
   DECLARE @V_OLD_CDEMPRESA    VARCHAR(2);
   DECLARE @V_OLD_CDINSCMUNI   VARCHAR(16);
   DECLARE @V_OLD_CDFILIAL     VARCHAR(4);
   DECLARE @V_OLD_NMREPRINSMUN VARCHAR(40);
   DECLARE @V_NEW_CDEMPRESA    VARCHAR(2);
   DECLARE @V_NEW_CDINSCMUNI   VARCHAR(16);
   DECLARE @V_NEW_CDFILIAL     VARCHAR(4);
   DECLARE @V_NEW_NMREPRINSMUN VARCHAR(40);

BEGIN
   SET @V_TIPO = '0';
   SET @V_DESC = '';
   SET @V_CDFILLOGOPER = '';

   EXEC NMUSUARIO  @V_USUARIO;
   SET @V_USUARIO = ISNULL(@V_USUARIO,'');

   DECLARE C_CURSOR CURSOR FOR
      SELECT
         'D' IDCOMANDO,
         CDEMPRESA,
         CDINSCMUNI,
         CDFILIAL,
         NMREPRINSMUN,
         NULL,
         NULL,
         NULL,
         NULL
      FROM
         DELETED
      WHERE
         CDEMPRESA+CDINSCMUNI NOT IN (SELECT CDEMPRESA+CDINSCMUNI FROM INSERTED)

      UNION ALL
      SELECT
         'U' IDCOMANDO,
         DELETED.CDEMPRESA,
         DELETED.CDINSCMUNI,
         DELETED.CDFILIAL,
         DELETED.NMREPRINSMUN,
         INSERTED.CDEMPRESA,
         INSERTED.CDINSCMUNI,
         INSERTED.CDFILIAL,
         INSERTED.NMREPRINSMUN
      FROM
         INSERTED,
         DELETED
      WHERE
         INSERTED.CDEMPRESA  = DELETED.CDEMPRESA   AND
         INSERTED.CDINSCMUNI = DELETED.CDINSCMUNI

      UNION ALL
      SELECT
         'I' IDCOMANDO,
         NULL,
         NULL,
         NULL,
         NULL,
         CDEMPRESA,
         CDINSCMUNI,
         CDFILIAL,
         NMREPRINSMUN
      FROM
         INSERTED
      WHERE
         CDEMPRESA+CDINSCMUNI NOT IN (SELECT CDEMPRESA+CDINSCMUNI FROM DELETED);

   OPEN C_CURSOR;
   FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                                 @V_OLD_CDEMPRESA,
                                 @V_OLD_CDINSCMUNI,
                                 @V_OLD_CDFILIAL,
                                 @V_OLD_NMREPRINSMUN,
                                 @V_NEW_CDEMPRESA,
                                 @V_NEW_CDINSCMUNI,
                                 @V_NEW_CDFILIAL,
                                 @V_NEW_NMREPRINSMUN;

   WHILE (@@FETCH_STATUS = 0) BEGIN
      IF @V_ID_COMANDO = 'D' /*DELETING*/ BEGIN
         IF @V_OLD_CDEMPRESA IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' CDEmpresa=' + @V_OLD_CDEMPRESA;
         END;

         IF @V_OLD_CDINSCMUNI IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' Cod=' + @V_OLD_CDINSCMUNI;
         END;

         IF @V_OLD_NMREPRINSMUN IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' ReprLegal=' + @V_OLD_NMREPRINSMUN;
         END;

         SET @V_TIPO         = '3';
         SET @V_ITPESQ       = @V_OLD_CDEMPRESA+@V_OLD_CDINSCMUNI;
         SET @V_CDFILLOGOPER = @V_OLD_CDFILIAL;
      END
      ELSE BEGIN
         IF @V_ID_COMANDO = 'U' /*UPDATING*/ BEGIN
            IF (@V_OLD_CDEMPRESA    <> @V_NEW_CDEMPRESA)    OR
               (@V_OLD_CDINSCMUNI   <> @V_NEW_CDINSCMUNI)   OR
               (ISNULL(@V_OLD_NMREPRINSMUN,'!') <> ISNULL(@V_NEW_NMREPRINSMUN,'!')) BEGIN
               SET @V_TIPO         = '2';
               SET @V_ITPESQ       = @V_OLD_CDEMPRESA+@V_OLD_CDINSCMUNI;
               SET @V_CDFILLOGOPER = @V_OLD_CDFILIAL;
            END;
         END
         ELSE BEGIN
            SET @V_TIPO         = '1';
            SET @V_ITPESQ       = @V_NEW_CDEMPRESA+@V_NEW_CDINSCMUNI;
            SET @V_CDFILLOGOPER = @V_NEW_CDFILIAL;
         END;


         IF @V_NEW_CDEMPRESA IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' Empresa=' + @V_NEW_CDEMPRESA;
         END;

         IF @V_NEW_CDINSCMUNI IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' Cod=' + @V_NEW_CDINSCMUNI;
         END;

         IF (ISNULL(@V_OLD_NMREPRINSMUN,'!') <> ISNULL(@V_NEW_NMREPRINSMUN,'!')) BEGIN
            SET @V_DESC = @V_DESC + ' ReprLegal=' + @V_NEW_NMREPRINSMUN;
         END;
      END;

      IF @V_TIPO <> '0' BEGIN
         EXEC NMUSUARIO @V_USUARIO;
         IF NOT @V_DESC IS NULL BEGIN
            IF NOT @V_USUARIO IS NULL BEGIN
               SET @V_DTOPER = GETDATE();
               INSERT INTO LOGOPERA
                  (NMTABELA,       DTOPERACAO, NMRESPOPER,
                   IDTIPOOPER,     CDPESQITEM, DSOPERACAO, CDFILLOGOPER)
               VALUES
                  ('INSCRMUNICIP', @V_DTOPER,  @V_USUARIO,
                   @V_TIPO,        @V_ITPESQ,  @V_DESC,    @V_CDFILLOGOPER);
            END;
         END;
      END;


      FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                                    @V_OLD_CDEMPRESA,
                                    @V_OLD_CDINSCMUNI,
                                    @V_OLD_CDFILIAL,
                                    @V_OLD_NMREPRINSMUN,
                                    @V_NEW_CDEMPRESA,
                                    @V_NEW_CDINSCMUNI,
                                    @V_NEW_CDFILIAL,
                                    @V_NEW_NMREPRINSMUN;
   END;
   CLOSE      C_CURSOR;
   DEALLOCATE C_CURSOR;
END;
GO


-- VERSAO --
EXEC ATUVEROBJ 'LOG_GER_INSCRMUNICIP','4.11.105';
GO
